"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.graphqlMiddleware = void 0;
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const mkdirp_1 = __importDefault(require("mkdirp"));
const chokidar_1 = __importDefault(require("chokidar"));
const consola_1 = __importDefault(require("consola"));
const serverMiddleware_1 = __importDefault(require("./../serverMiddleware"));
const graphqlImport_1 = __importDefault(require("./graphqlImport"));
const codegen_1 = __importDefault(require("./codegen"));
const logger = consola_1.default.withTag('nuxt-graphql-middleware');
const PLUGIN_PATH = path_1.default.resolve(__dirname, './../plugin/index.js');
var FileType;
(function (FileType) {
    FileType["Query"] = "query";
    FileType["Mutation"] = "mutation";
})(FileType || (FileType = {}));
function resolveGraphqlFile(file, resolver) {
    return fs_1.default.promises
        .readFile(file)
        .then((buffer) => buffer.toString())
        .then((source) => graphqlImport_1.default(source, resolver));
}
function writeSource(dest, filePath, source) {
    const fileName = path_1.default.basename(filePath);
    const out = path_1.default.resolve(dest, fileName);
    return fs_1.default.promises.writeFile(out, source);
}
function resolveGraphql(files, map, resolver, filesMap, type, outputPath) {
    return Promise.all(Object.keys(files).map((name) => {
        const filePath = files[name];
        const file = resolver(filePath);
        return resolveGraphqlFile(file, resolver).then((source) => {
            map.set(name, source);
            if (outputPath) {
                writeSource(outputPath, filePath, source);
            }
            filesMap.set(file, {
                type,
                name,
                file: filePath,
            });
        });
    }));
}
const graphqlMiddleware = function () {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s;
    return __awaiter(this, void 0, void 0, function* () {
        const resolver = this.nuxt.resolver.resolveAlias;
        const options = this.options;
        const provided = (this.options.graphqlMiddleware ||
            {});
        const config = {
            graphqlServer: provided.graphqlServer || '',
            typescript: {
                enabled: !!((_a = provided.typescript) === null || _a === void 0 ? void 0 : _a.enabled),
                schemaOptions: (_b = provided.typescript) === null || _b === void 0 ? void 0 : _b.schemaOptions,
                resolvedQueriesPath: ((_c = provided.typescript) === null || _c === void 0 ? void 0 : _c.resolvedQueriesPath) || provided.outputPath || '',
                schemaOutputPath: ((_d = provided.typescript) === null || _d === void 0 ? void 0 : _d.schemaOutputPath) || '~/schema',
                typesOutputPath: ((_e = provided.typescript) === null || _e === void 0 ? void 0 : _e.typesOutputPath) || '~/types',
                skipSchemaDownload: !!((_f = provided.typescript) === null || _f === void 0 ? void 0 : _f.skipSchemaDownload),
            },
            endpointNamespace: provided.endpointNamespace || '/__graphql_middleware',
            debug: provided.debug || options.dev,
            queries: provided.queries || {},
            mutations: provided.mutations || {},
            outputPath: provided.outputPath || '',
            server: provided.server,
            plugin: {
                enabled: !!((_g = provided.plugin) === null || _g === void 0 ? void 0 : _g.enabled),
                cacheInBrowser: !!((_h = provided.plugin) === null || _h === void 0 ? void 0 : _h.cacheInBrowser),
                cacheInServer: !!((_j = provided.plugin) === null || _j === void 0 ? void 0 : _j.cacheInServer),
            },
        };
        if ((_k = config.plugin) === null || _k === void 0 ? void 0 : _k.enabled) {
            this.addPlugin({
                src: PLUGIN_PATH,
                options: {
                    namespace: config.endpointNamespace,
                    cacheInBrowser: ((_l = config.plugin) === null || _l === void 0 ? void 0 : _l.cacheInBrowser) ? 'true' : 'false',
                    cacheInServer: ((_m = config.plugin) === null || _m === void 0 ? void 0 : _m.cacheInServer) ? 'true' : 'false',
                },
            });
        }
        const fileMap = new Map();
        const queries = new Map();
        const mutations = new Map();
        const outputPath = config.outputPath ? resolver(config.outputPath) : '';
        yield mkdirp_1.default(outputPath);
        const schemaOutputPath = resolver((_o = config.typescript) === null || _o === void 0 ? void 0 : _o.schemaOutputPath);
        const typesOutputPath = resolver((_p = config.typescript) === null || _p === void 0 ? void 0 : _p.typesOutputPath);
        const { generateSchema, generateTypes } = codegen_1.default(config.graphqlServer, {
            resolvedQueriesPath: config.outputPath,
            schemaOptions: (_q = config.typescript) === null || _q === void 0 ? void 0 : _q.schemaOptions,
            schemaOutputPath,
            typesOutputPath,
        });
        if (((_r = config.typescript) === null || _r === void 0 ? void 0 : _r.enabled) && !((_s = config.typescript) === null || _s === void 0 ? void 0 : _s.skipSchemaDownload)) {
            if (!outputPath) {
                throw new Error('TypeScript enabled, but no outputPath given.');
            }
            yield mkdirp_1.default(schemaOutputPath);
            yield generateSchema();
        }
        function build() {
            logger.log('Building GraphQL files...');
            return Promise.all([
                resolveGraphql(config.queries, queries, resolver, fileMap, FileType.Query, outputPath),
                resolveGraphql(config.mutations, mutations, resolver, fileMap, FileType.Mutation, outputPath),
            ]).then(() => {
                var _a;
                logger.success('Finished building GraphQL files');
                if ((_a = config.typescript) === null || _a === void 0 ? void 0 : _a.enabled) {
                    return generateTypes().then(() => {
                        logger.success('Finished generating GraphQL TypeScript files.');
                    });
                }
            });
        }
        function watchFiles() {
            const ignored = ['node_modules', '.nuxt'];
            if (config.outputPath) {
                ignored.push(config.outputPath);
            }
            const filesWatcher = chokidar_1.default.watch('./**/*.graphql', {
                ignoreInitial: true,
                ignored,
            });
            if (filesWatcher) {
                logger.info('Watching for query changes');
                filesWatcher.on('change', () => {
                    build();
                });
            }
            return filesWatcher;
        }
        let watcher;
        if (this.nuxt.options.dev) {
            this.nuxt.hook('build:done', () => {
                watcher = watchFiles();
            });
            this.nuxt.hook('close', () => {
                if (watcher) {
                    watcher.close();
                    watcher = undefined;
                }
            });
        }
        build().then(() => {
            if (options.debug) {
                logger.info('Available queries and mutations:');
                console.table(Array.from(fileMap.entries()).map(([_key, value]) => value));
            }
        });
        this.addServerMiddleware({
            path: config.endpointNamespace,
            handler: serverMiddleware_1.default(config.graphqlServer, queries, mutations, config.server),
        });
    });
};
exports.graphqlMiddleware = graphqlMiddleware;
